<?php
/**
 * Déclarations relatives à la base de données
 *
 * @plugin     vrdv
 * @copyright  2024
 * @author     Vincent CALLIES
 * @licence    GNU/GPL
 * @package    SPIP\Rdv\Base
 */

if (!defined('_ECRIRE_INC_VERSION')) {
	return;
}


/**
 * Déclaration des alias de tables et filtres automatiques de champs
 *
 * Le plugin
 * - déclare les alias de tables des calendriers et des rendez-vous
 * - déclare un traitement pour la description des rendez-vous
 *   @link : https://programmer.spip.net/Traitements-automatiques-des
 *
 * @pipeline declarer_tables_interfaces
 *
 * @param array $interfaces
 *     Déclarations d'interface pour le compilateur
 * @return array
 *     Déclarations d'interface pour le compilateur
 */
function vrdv_declarer_tables_interfaces($interfaces) {

	$interfaces['table_des_tables']['vrdvs'] = 'vrdvs';
	$interfaces['table_des_tables']['vcalendriers'] = 'vcalendriers';
	$interfaces['table_des_traitements']['DESCRIPTION']['vrdvs'] = _TRAITEMENT_RACCOURCIS;
	
	return $interfaces;
}


/**
 * Déclaration des objets éditoriaux
 *
 * Création de deux objets :
 * - des rendez-vous
 * Une instanciation limitée à publie|archive|anonyme|poubelle.
 * @link https://contrib.spip.net/Objet-archiver-anonymiser
 * Pas de gestion de version,
 * ni de gestion de multilinguisme.
 * La création doit restée simple et économe en énergie.
 * - des calendriers
 * Chaque rdv a pour parent explicite un calendrier.
 *
 * @pipeline declarer_tables_objets_sql
 * @param array $tables
 *     Description des tables
 * @return array
 *     Description complétée des tables
 */
function vrdv_declarer_tables_objets_sql($tables) {

	$tables['spip_vrdvs'] = [
		'type' => 'vrdv',
		'table_objet' => 'vrdvs',
		'table_objet_surnoms' => [],
		'principale' => 'oui',
		'field' => [
			'id_vrdv' => 'bigint(21) NOT NULL',
			'id_vcalendrier' => 'bigint(21) DEFAULT "0" NOT NULL',
			'date_debut' => 'datetime NOT NULL DEFAULT "0000-00-00 00:00:00"',
			'date_fin' => 'datetime NOT NULL DEFAULT "0000-00-00 00:00:00"',
			'toute_la_journee' => 'varchar(3) DEFAULT "non" NOT NULL',
			'recurrence_jour' => 'varchar(1) DEFAULT "0" NOT NULL',
			'recurrence_semaine' => 'varchar(1) DEFAULT "0" NOT NULL',
			'titre' => 'tinytext NOT NULL DEFAULT ""',
			'description' => 'text NOT NULL DEFAULT ""',
			'statut' => 'varchar(10) DEFAULT "0" NOT NULL',
			'maj' => 'TIMESTAMP',
		],
		'key' => [
			'PRIMARY KEY' => 'id_vrdv',
			'KEY id_vcalendrier' => 'id_vcalendrier',
			'KEY date_debut' => 'date_debut',
			'KEY date_fin' => 'date_fin',
			'KEY toute_la_journee' => 'toute_la_journee',
			'KEY recurrence_jour' => 'recurrence_jour',
			'KEY recurrence_semaine' => 'recurrence_semaine',
			'KEY statut' => 'statut',
		],
		'champs_contenu' => [
			'id_vcalendrier',
			'date_debut',
			'date_fin', 
			'toute_la_journee',
			'recurrence_jour',
			'recurrence_semaine',
			#'titre', // titre
			'description', 
		],
		'champs_editables'  => [
			'id_vcalendrier',
			'date_debut',
			'date_fin',
			'toute_la_journee',
			'recurrence_jour',
			'recurrence_semaine',
			'titre',
			'description',
		],
		'champs_anonymises' => [
			'titre' => 'void', // rien
			'description' => 'void', // rien
		],
		'parent' => [
			[
				'type' => 'vcalendrier',
				'champ' => 'id_vcalendrier'
			]
		],
		'titre' => 'titre AS titre',
		#'date' => '',  // indique le nom du field pour le formulaires_dater_charger_dist
		'rechercher_champs' => [
			'titre' => 8,
			'description' => 3
		],
		'statut' => [
			[
				'champ' => 'statut',
				'publie' => 'publie',
				'previsu' => '',
				'post_date' => '',
				'exception' => ['statut', 'tout']
			]
		],
		'statut_titres' => [
			'publie' => 'vrdv:info_vrdv_publie',
			'archive' => 'vrdv:info_vrdv_archive',
			'anonyme' => 'objet_archiver_anonymiser:info_anonymise',
			'poubelle' => 'objet_archiver_anonymiser:info_archive'
		],
		'statut_textes_instituer' => [
			'publie' => 'texte_statut_publie',
			'archive' => 'objet_archiver_anonymiser:statut_archive',
			'anonyme' => 'objet_archiver_anonymiser:statut_anonymise',
			'poubelle' => 'texte_statut_poubelle',
		],
		'statut_images' => [
			'publie'   => 'puce-publier-xx.svg',
			'archive' => 'puce-archiver2-xx.svg',
			'anonyme' => 'puce-anonymiser-xx.svg',
			'poubelle' => 'puce-supprimer-xx.svg',
		],
		'texte_changer_statut' => 'vrdv:texte_changer_statut',
		'aide_changer_statut' => 'vrdv_statut',
		'tables_jointures'  => [
			'spip_rdvs_liens'
		],
		'texte_retour'       => 'icone_retour',
		'texte_objets'       => 'vrdv:titre_vrdvs',
		'texte_objet'        => 'vrdv:titre_vrdv',
		'texte_modifier'     => 'vrdv:icone_modifier_vrdv',
		'texte_creer'        => 'vrdv:icone_creer_vrdv',
		'info_aucun_objet'   => 'vrdv:info_aucun_vrdv',
		'info_1_objet'       => 'vrdv:info_1_vrdv',
		'info_nb_objets'     => 'vrdv:info_nb_vrdvs',
		'texte_logo_objet'   => 'vrdv:titre_logo_vrdv',

		// prévoit une gestion des rôles mais n'en fournit pas
		'roles_colonne' => 'role',
		'roles_titres'  => [],
		'roles_objets'  => [],
	];

	$tables['spip_vcalendriers'] = [
		'type' => 'vcalendrier',
		'table_objet' => 'vcalendriers',
		'table_objet_surnoms' => [],
		'principale' => 'oui',
		'field' => [
			'id_vcalendrier' => 'bigint(21) NOT NULL',
			'titre' => 'tinytext NOT NULL DEFAULT ""',
			'abreviation' => 'tinytext NOT NULL DEFAULT ""',
			'maj' => 'TIMESTAMP',
		],
		'key' => [
			'PRIMARY KEY' => 'id_vcalendrier',
		],
		'champs_contenu' => [
			#'titre',
			'abreviation', 
		],
		'champs_editables'  => [
			'titre',
			'abreviation',
		],
		'rechercher_champs' => ['titre' => 8, 'abreviation' => 3],
		'tables_jointures'  => [
			'vcalendriers_liens',
			'id_auteur' => 'auteurs_liens'
		],
		'titre' => 'titre AS titre',
		#'date' => 'date_debut',  // indique le nom du field pour le formulaires_dater_charger_dist
		'texte_retour'       => 'icone_retour',
		'texte_objets'       => 'vcalendrier:titre_vcalendriers',
		'texte_objet'        => 'vcalendrier:titre_vcalendrier',
		'texte_modifier'     => 'vcalendrier:icone_modifier_vcalendrier',
		'texte_creer'        => 'vcalendrier:icone_creer_vcalendrier',
		'info_aucun_objet'   => 'vcalendrier:info_aucun_vcalendrier',
		'info_1_objet'       => 'vcalendrier:info_1_vcalendrier',
		'info_nb_objets'     => 'vcalendrier:info_nb_vcalendriers',
		'texte_logo_objet'   => 'vcalendrier:titre_logo_vcalendrier',

		// prévoit une gestion des rôles mais n'en fournit pas
		'roles_colonne' => 'role',
		'roles_titres'  => [],
		'roles_objets'  => []
		];

	return $tables;
}

/**
 * Déclaration des tables auxiliaires
 *
 * Généralement les tables auxiliaires sont des tables de laisons
 * permettant d'associer des objets à d'autres objets.
 *
 */
function vrdv_declarer_tables_auxiliaires($tables) {

	$tables['spip_vrdvs_liens'] = [
		'field' => [
			'id_vrdv' => "bigint(21) NOT NULL DEFAULT '0'",
			'id_objet' => "bigint(21) DEFAULT '0' NOT NULL",
			'objet' => "varchar(25) DEFAULT '' NOT NULL",
			'role' => "varchar(30) NOT NULL DEFAULT ''",
		],
		'key' => [
			'PRIMARY KEY' => 'id_vrdv,id_objet,objet,role',
			'KEY id_vrdv' => 'id_vrdv',
			'KEY id_objet' => 'id_objet',
			'KEY objet' => 'objet',
			'KEY role' => 'role',
		]
	];

	$tables['spip_vcalendriers_liens'] = [
		'field' => [
			'id_vcalendrier' => "bigint(21) NOT NULL DEFAULT '0'",
			'id_objet' => "bigint(21) DEFAULT '0' NOT NULL",
			'objet' => "varchar(25) DEFAULT '' NOT NULL",
			'role' => "varchar(30) NOT NULL DEFAULT ''",
		],
		'key' => [
			'PRIMARY KEY' => 'id_vcalendrier,id_objet,objet,role',
			'KEY id_vcalendrier' => 'id_vcalendrier',
			'KEY id_objet' => 'id_objet',
			'KEY objet' => 'objet',
			'KEY role' => 'role',
		],
	];

	return $tables;
}
